""" Unit tests for Py-ART's correct/filters.py module. """

import netCDF4
import numpy as np
import pytest

import pyart

radar = pyart.testing.make_empty_ppi_radar(10, 36, 1)

# a simple field
fdata = np.tile(np.arange(10.0), 36).reshape(36, 10)
radar.add_field("test_field", {"data": fdata})

# more
fdata2 = np.ma.masked_array(fdata, copy=True)
fdata2[2, 2] = np.ma.masked
fdata2[3, 3] = np.NAN
fdata2[4, 4] = np.PINF
fdata2[5, 5] = np.NINF
radar.add_field("test_field2", {"data": fdata2})


def test_gatefilter_init():
    gfilter = pyart.correct.GateFilter(radar)
    assert np.all(gfilter.gate_excluded == np.False_)


def test_gatefilter_copy():
    gfilter = pyart.correct.GateFilter(radar)
    gfilter2 = gfilter.copy()
    gfilter.exclude_below("test_field", 5)
    assert gfilter.gate_excluded[0, 0] is np.True_
    assert gfilter.gate_excluded[0, -1] is np.False_

    assert gfilter2.gate_excluded[0, 0] is np.False_
    assert gfilter2.gate_excluded[0, -1] is np.False_


def test_exclude_transition():

    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_transition()
    assert np.all(gfilter.gate_excluded == np.False_)

    trans_radar = pyart.testing.make_empty_ppi_radar(12, 5, 1)
    trans_radar.antenna_transition = {"data": np.array([1, 0, 0, 0, 1])}
    gfilter = pyart.correct.GateFilter(trans_radar)
    gfilter.exclude_transition()

    assert gfilter.gate_excluded[0, 0] is np.True_
    assert gfilter.gate_excluded[0, 1] is np.True_
    assert gfilter.gate_excluded[-1, 0] is np.True_
    assert gfilter.gate_excluded[-1, 1] is np.True_
    assert gfilter.gate_excluded[1, 0] is np.False_
    assert gfilter.gate_excluded[1, 1] is np.False_


def test_include_not_transition():

    gfilter = pyart.correct.GateFilter(radar, exclude_based=False)
    gfilter.include_not_transition()
    assert np.all(gfilter.gate_excluded == np.False_)

    trans_radar = pyart.testing.make_empty_ppi_radar(12, 5, 1)
    trans_radar.antenna_transition = {"data": np.array([1, 0, 0, 0, 1])}
    gfilter = pyart.correct.GateFilter(trans_radar, exclude_based=False)
    gfilter.include_not_transition()

    assert gfilter.gate_excluded[0, 0] is np.True_
    assert gfilter.gate_excluded[0, 1] is np.True_
    assert gfilter.gate_excluded[-1, 0] is np.True_
    assert gfilter.gate_excluded[-1, 1] is np.True_
    assert gfilter.gate_excluded[1, 0] is np.False_
    assert gfilter.gate_excluded[1, 1] is np.False_


def test_gatefilter_exclude_below():
    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_below("test_field", 5)
    assert gfilter.gate_excluded[0, 0] is np.True_
    assert gfilter.gate_excluded[0, -1] is np.False_

    assert gfilter.gate_excluded[0, 5] is np.False_
    gfilter.exclude_below("test_field", 5, inclusive=True)
    assert gfilter.gate_excluded[0, 5] is np.True_

    gfilter.exclude_below("test_field", 99)
    assert gfilter.gate_excluded[0, 0] is np.True_
    assert gfilter.gate_excluded[0, -1] is np.True_


def test_gatefilter_exclude_above():
    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_above("test_field", 5)
    assert gfilter.gate_excluded[0, 0] is np.False_
    assert gfilter.gate_excluded[0, -1] is np.True_

    assert gfilter.gate_excluded[0, 5] is np.False_
    gfilter.exclude_above("test_field", 5, inclusive=True)
    assert gfilter.gate_excluded[0, 5] is np.True_

    gfilter.exclude_above("test_field", -5)
    assert gfilter.gate_excluded[0, 0] is np.True_
    assert gfilter.gate_excluded[0, -1] is np.True_


def test_gatefilter_exclude_inside():
    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_inside("test_field", 2, 5, inclusive=False)
    assert gfilter.gate_excluded[0, 0] is np.False_
    assert gfilter.gate_excluded[0, 3] is np.True_
    assert gfilter.gate_excluded[0, -1] is np.False_

    assert gfilter.gate_excluded[0, 2] is np.False_
    gfilter.exclude_inside("test_field", 2, 5, inclusive=True)
    assert gfilter.gate_excluded[0, 2] is np.True_

    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_inside("test_field", 5, 2)
    assert gfilter.gate_excluded[0, 0] is np.False_
    assert gfilter.gate_excluded[0, 3] is np.True_
    assert gfilter.gate_excluded[0, -1] is np.False_


def test_gatefilter_exclude_outside():
    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_outside("test_field", 2, 5)
    assert gfilter.gate_excluded[0, 0] is np.True_
    assert gfilter.gate_excluded[0, 3] is np.False_
    assert gfilter.gate_excluded[0, -1] is np.True_

    assert gfilter.gate_excluded[0, 2] is np.False_
    gfilter.exclude_outside("test_field", 2, 5, inclusive=True)
    assert gfilter.gate_excluded[0, 2] is np.True_

    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_outside("test_field", 5, 2)
    assert gfilter.gate_excluded[0, 0] is np.True_
    assert gfilter.gate_excluded[0, 3] is np.False_
    assert gfilter.gate_excluded[0, -1] is np.True_


def test_gatefilter_exclude_equal():
    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_equal("test_field", 2)
    assert gfilter.gate_excluded[0, 2] is np.True_
    assert gfilter.gate_excluded[0, 3] is np.False_


def test_gatefilter_exclude_not_equal():
    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_not_equal("test_field", 2)
    assert gfilter.gate_excluded[0, 2] is np.False_
    assert gfilter.gate_excluded[0, 3] is np.True_


def test_gatefilter_exclude_all():
    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_all()
    assert np.all(gfilter.gate_excluded == np.True_)


def test_gatefilter_exclude_none():
    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_none()
    assert np.all(gfilter.gate_excluded == np.False_)


def test_gatefilter_exclude_masked():
    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_masked("test_field2")
    assert gfilter.gate_excluded[0, 0] is np.False_
    assert gfilter.gate_excluded[2, 2] is np.True_
    assert gfilter.gate_excluded[3, 3] is np.False_
    assert gfilter.gate_excluded[4, 4] is np.False_
    assert gfilter.gate_excluded[5, 5] is np.False_


def test_gatefilter_exclude_invalid():
    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_invalid("test_field2")
    assert gfilter.gate_excluded[0, 0] is np.False_
    assert gfilter.gate_excluded[2, 2] is np.True_
    assert gfilter.gate_excluded[3, 3] is np.True_
    assert gfilter.gate_excluded[4, 4] is np.True_
    assert gfilter.gate_excluded[5, 5] is np.True_


def test_gatefilter_exclude_gates():
    gfilter = pyart.correct.GateFilter(radar)
    gfilter.include_all()
    gates = gfilter.gate_excluded
    gates[2, 0] = np.True_
    gates[2, 2] = np.True_
    gfilter.exclude_gates(gates)
    # exclude when included
    assert gfilter.gate_excluded[0, 0] is np.False_
    assert gfilter.gate_excluded[2, 0] is np.True_
    # exclude when already excluded
    gfilter.exclude_gates(gates)
    assert gfilter.gate_excluded[2, 0] is np.True_
    gates[2, 0] = np.False_
    gates[0, 2] = np.True_
    # exclude with op='and'
    gfilter.exclude_gates(gates, op="and")
    assert gfilter.gate_excluded[2, 0] is np.False_
    assert gfilter.gate_excluded[0, 2] is np.False_
    assert gfilter.gate_excluded[2, 2] is np.True_


def test_gatefilter_ops():
    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_below("test_field", 0.5, op="or")
    assert gfilter.gate_excluded[0, 0] is np.True_
    assert gfilter.gate_excluded[0, 1] is np.False_
    assert gfilter.gate_excluded[0, 9] is np.False_
    gfilter.exclude_above("test_field", 8.5, op="or")
    assert gfilter.gate_excluded[0, 0] is np.True_
    assert gfilter.gate_excluded[0, 1] is np.False_
    assert gfilter.gate_excluded[0, 9] is np.True_

    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_below("test_field", 0.5, op="or")
    assert gfilter.gate_excluded[0, 0] is np.True_
    assert gfilter.gate_excluded[0, 1] is np.False_
    assert gfilter.gate_excluded[0, 9] is np.False_
    gfilter.exclude_above("test_field", 8.5, op="and")
    assert gfilter.gate_excluded[0, 0] is np.False_
    assert gfilter.gate_excluded[0, 1] is np.False_
    assert gfilter.gate_excluded[0, 9] is np.False_

    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_below("test_field", 0.5, op="or")
    assert gfilter.gate_excluded[0, 0] is np.True_
    assert gfilter.gate_excluded[0, 1] is np.False_
    assert gfilter.gate_excluded[0, 9] is np.False_
    gfilter.exclude_above("test_field", 8.5, op="new")
    assert gfilter.gate_excluded[0, 0] is np.False_
    assert gfilter.gate_excluded[0, 1] is np.False_
    assert gfilter.gate_excluded[0, 9] is np.True_


def test_gatefilter_raises():
    gfilter = pyart.correct.GateFilter(radar)
    pytest.raises(ValueError, gfilter.exclude_below, "test_field", 0.5, op="fuzz")
    pytest.raises(
        ValueError, gfilter.exclude_below, "test_field", 0.5, exclude_masked="fuzz"
    )
    pytest.raises(ValueError, gfilter.exclude_gates, np.arange(2))
    pytest.raises(ValueError, gfilter.include_gates, np.arange(2))


#################
# include tests #
#################


def test_gatefilter_gate_included_attribute():
    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_below("test_field", 0.5, op="or")
    assert gfilter.gate_excluded[0, 0] is np.True_
    assert gfilter.gate_excluded[0, 1] is np.False_
    assert gfilter.gate_included[0, 0] is np.False_
    assert gfilter.gate_included[0, 1] is np.True_


def test_gatefilter_include_below():
    gfilter = pyart.correct.GateFilter(radar, exclude_based=False)
    gfilter.include_below("test_field", 5)
    assert gfilter.gate_included[0, 0] is np.True_
    assert gfilter.gate_included[0, -1] is np.False_

    assert gfilter.gate_included[0, 5] is np.False_
    gfilter.include_below("test_field", 5, inclusive=True)
    assert gfilter.gate_included[0, 5] is np.True_

    gfilter.include_below("test_field", 99)
    assert gfilter.gate_included[0, 0] is np.True_
    assert gfilter.gate_included[0, -1] is np.True_


def test_gatefilter_include_above():
    gfilter = pyart.correct.GateFilter(radar, exclude_based=False)
    gfilter.include_above("test_field", 5)
    assert gfilter.gate_included[0, 0] is np.False_
    assert gfilter.gate_included[0, -1] is np.True_

    assert gfilter.gate_included[0, 5] is np.False_
    gfilter.include_above("test_field", 5, inclusive=True)
    assert gfilter.gate_included[0, 5] is np.True_

    gfilter.include_above("test_field", -5)
    assert gfilter.gate_included[0, 0] is np.True_
    assert gfilter.gate_included[0, -1] is np.True_


def test_gatefilter_include_inside():
    gfilter = pyart.correct.GateFilter(radar, exclude_based=False)
    gfilter.include_inside("test_field", 2, 5, inclusive=False)
    assert gfilter.gate_included[0, 0] is np.False_
    assert gfilter.gate_included[0, 3] is np.True_
    assert gfilter.gate_included[0, -1] is np.False_

    assert gfilter.gate_included[0, 2] is np.False_
    gfilter.include_inside("test_field", 2, 5, inclusive=True)
    assert gfilter.gate_included[0, 2] is np.True_

    gfilter = pyart.correct.GateFilter(radar, exclude_based=False)
    gfilter.include_inside("test_field", 5, 2)
    assert gfilter.gate_included[0, 0] is np.False_
    assert gfilter.gate_included[0, 3] is np.True_
    assert gfilter.gate_included[0, -1] is np.False_


def test_gatefilter_include_outside():
    gfilter = pyart.correct.GateFilter(radar, exclude_based=False)
    gfilter.include_outside("test_field", 2, 5)
    assert gfilter.gate_included[0, 0] is np.True_
    assert gfilter.gate_included[0, 3] is np.False_
    assert gfilter.gate_included[0, -1] is np.True_

    assert gfilter.gate_included[0, 2] is np.False_
    gfilter.include_outside("test_field", 2, 5, inclusive=True)
    assert gfilter.gate_included[0, 2] is np.True_

    gfilter = pyart.correct.GateFilter(radar, exclude_based=False)
    gfilter.include_outside("test_field", 5, 2)
    assert gfilter.gate_included[0, 0] is np.True_
    assert gfilter.gate_included[0, 3] is np.False_
    assert gfilter.gate_included[0, -1] is np.True_


def test_gatefilter_include_equal():
    gfilter = pyart.correct.GateFilter(radar, exclude_based=False)
    gfilter.include_equal("test_field", 2)
    assert gfilter.gate_included[0, 2] is np.True_
    assert gfilter.gate_included[0, 3] is np.False_


def test_gatefilter_include_not_equal():
    gfilter = pyart.correct.GateFilter(radar, exclude_based=False)
    gfilter.include_not_equal("test_field", 2)
    assert gfilter.gate_included[0, 2] is np.False_
    assert gfilter.gate_included[0, 3] is np.True_


def test_gatefilter_include_all():
    gfilter = pyart.correct.GateFilter(radar, exclude_based=False)
    gfilter.include_all()
    assert np.all(gfilter.gate_included == np.True_)


def test_gatefilter_include_none():
    gfilter = pyart.correct.GateFilter(radar, exclude_based=False)
    gfilter.include_none()
    assert np.all(gfilter.gate_included == np.False_)


def test_gatefilter_include_masked():
    gfilter = pyart.correct.GateFilter(radar, exclude_based=False)
    gfilter.include_not_masked("test_field2")
    assert gfilter.gate_included[0, 0] is np.True_
    assert gfilter.gate_included[2, 2] is np.False_
    assert gfilter.gate_included[3, 3] is np.True_
    assert gfilter.gate_included[4, 4] is np.True_
    assert gfilter.gate_included[5, 5] is np.True_


def test_gatefilter_include_valid():
    gfilter = pyart.correct.GateFilter(radar, exclude_based=False)
    gfilter.include_valid("test_field2")
    assert gfilter.gate_included[0, 0] is np.True_
    assert gfilter.gate_included[2, 2] is np.False_
    assert gfilter.gate_included[3, 3] is np.False_
    assert gfilter.gate_included[4, 4] is np.False_
    assert gfilter.gate_included[5, 5] is np.False_


def test_gatefilter_include_gates():
    gfilter = pyart.correct.GateFilter(radar)
    gfilter.exclude_all()
    gates = gfilter.gate_included
    gates[2, 0] = np.True_
    gates[2, 2] = np.True_
    gfilter.include_gates(gates)
    # include when excluded
    assert gfilter.gate_included[0, 0] is np.False_
    assert gfilter.gate_included[2, 0] is np.True_
    # include when already included
    gfilter.include_gates(gates)
    assert gfilter.gate_included[2, 0] is np.True_
    gates[2, 0] = np.False_
    gates[0, 2] = np.True_
    # include with op='or'
    gfilter.include_gates(gates, op="or")
    assert gfilter.gate_included[2, 0] is np.False_
    assert gfilter.gate_included[0, 2] is np.False_
    assert gfilter.gate_included[2, 2] is np.True_


radar2 = pyart.io.read(pyart.testing.NEXRAD_ARCHIVE_MSG31_FILE)


def test_moment_based_gate_filter():
    fdata = np.full((7200, 1832), 0.5)
    radar2.add_field("normalized_coherent_power", {"data": fdata})

    gfilter = pyart.correct.moment_based_gate_filter(
        radar2, min_ncp=0.4, min_refl=-33.0, max_refl=-31.0, min_rhv=0.207
    )
    gfilter2 = pyart.correct.moment_based_gate_filter(
        radar2, min_ncp=0.6, min_refl=-31.0, max_refl=-28.0, min_rhv=0.209
    )

    assert gfilter.gate_excluded[0, 0] is np.False_
    assert gfilter.gate_excluded[0, -1] is np.True_

    assert gfilter2.gate_excluded[0, 0] is np.True_
    assert gfilter2.gate_excluded[0, -1] is np.True_


sonde = netCDF4.Dataset(pyart.testing.SONDE_FILE)
radar3 = pyart.io.read(pyart.testing.NEXRAD_ARCHIVE_MSG31_COMPRESSED_FILE)
z_dict, temp_dict = pyart.retrieve.map_profile_to_gates(
    sonde.variables["tdry"][:], sonde.variables["alt"][:], radar3, toa=16500
)
radar3.add_field("temperature", temp_dict, replace_existing=True)
radar3.add_field("height", z_dict, replace_existing=True)


def test_moment_and_texture_based_gate_filter():
    gfilter = pyart.filters.moment_and_texture_based_gate_filter(
        radar3,
        phi_field="differential_phase",
        max_textphi=20.0,
        max_textrhv=0.3,
        max_textzdr=2.85,
        max_textrefl=8.0,
        min_rhv=0.6,
    )

    assert gfilter.gate_excluded[0, 0] is np.True_
    assert gfilter.gate_excluded[0, -1] is np.True_

    gfilter2 = pyart.filters.moment_and_texture_based_gate_filter(
        radar3,
        phi_field="differential_phase",
        max_textphi=280.0,
        max_textrhv=0.9,
        max_textzdr=10,
        max_textrefl=33.0,
        min_rhv=0.2,
    )

    assert gfilter2.gate_excluded[0, 0] is np.False_
    assert gfilter2.gate_excluded[0, -1] is np.True_


def test_temp_based_gate_filter():
    gfilter = pyart.filters.temp_based_gate_filter(radar3, min_temp=17.0)

    assert gfilter.gate_excluded[0, 0] is np.True_
    assert gfilter.gate_excluded[0, -1] is np.True_

    gfilter2 = pyart.filters.temp_based_gate_filter(
        radar3, thickness=None, min_temp=17.0
    )

    assert gfilter2.gate_excluded[0, 27] is np.False_
    assert gfilter2.gate_excluded[0, -1] is np.True_


def test_iso0_based_gate_filter():
    gfilter = pyart.filters.iso0_based_gate_filter(
        radar3, iso0_field="height", max_h_iso0=None, thickness=None, beamwidth=1.0
    )

    assert gfilter.gate_excluded[0, 0] is np.False_
    assert gfilter.gate_excluded[0, -1] is np.False_

    gfilter2 = pyart.filters.iso0_based_gate_filter(
        radar3, iso0_field="height", max_h_iso0=1000.0, thickness=400.0, beamwidth=None
    )

    assert gfilter2.gate_excluded[0, 0] is np.False_
    assert gfilter2.gate_excluded[0, -1] is np.True_



"""
Unit Tests for Py-ART's graph/radar_display.py module.
Execute this script to create figure_plot_radar_display_*.png files.
TODO use matplotlib's @image_comparison decorator to compare to file
in baseline_images directory. Current this test only determines if files can
be created, not that they are correct.
"""

import warnings

import cftime
import matplotlib.pyplot as plt
import numpy as np
import pytest
from numpy.testing import assert_almost_equal

import pyart


# Top level Figure generating tests
def test_radardisplay_rhi(outfile=None):
    radar = pyart.io.read_cfradial(pyart.testing.CFRADIAL_RHI_FILE)
    display = pyart.graph.RadarDisplay(radar)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    display.plot("reflectivity_horizontal", 0, ax=ax, mask_outside=True)
    if outfile:
        fig.savefig(outfile)
    plt.close()


def test_radardisplay_ppi(outfile=None):
    radar = pyart.io.read_cfradial(pyart.testing.CFRADIAL_PPI_FILE)
    radar.antenna_transition = {"data": np.zeros(radar.nrays)}
    display = pyart.graph.RadarDisplay(radar, shift=(0.1, 0.0))
    fig = plt.figure()
    ax = fig.add_subplot(111)

    gatefilter = pyart.filters.GateFilter(radar)
    display.plot(
        "reflectivity_horizontal",
        0,
        colorbar_flag=True,
        title="Fancy PPI Plot",
        mask_outside=True,
        mask_tuple=("reflectivity_horizontal", -100),
        gatefilter=gatefilter,
    )
    display.plot_colorbar()
    display.plot_range_rings([10, 20, 30, 40], ax=ax)
    display.plot_labels(["tree"], [(36.68, -97.62)], symbols="k+", ax=ax)
    display.plot_cross_hair(2)
    display.plot_grid_lines()
    display.set_aspect_ratio()
    display.set_limits(ylim=[-50, 50], xlim=[-50, 50])
    if outfile:
        fig.savefig(outfile)
    plt.close()


def test_radardisplay_ray(outfile=None):
    radar = pyart.io.read_cfradial(pyart.testing.CFRADIAL_RHI_FILE)
    display = pyart.graph.RadarDisplay(radar)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    display.plot_ray(
        "reflectivity_horizontal",
        0,
        ray_min=15,
        ray_max=40,
        mask_outside=True,
        ax=ax,
        mask_tuple=("reflectivity_horizontal", -10),
    )
    if outfile:
        fig.savefig(outfile)
    plt.close()


def test_radardisplay_vpt(outfile=None):
    radar = pyart.io.read_cfradial(pyart.testing.CFRADIAL_PPI_FILE)
    radar.antenna_transition = {"data": np.zeros(radar.nrays)}
    pyart.util.to_vpt(radar)  # hack to make the data a VPT
    display = pyart.graph.RadarDisplay(radar)
    fig = plt.figure()
    ax = fig.add_subplot(111)

    gatefilter = pyart.filters.GateFilter(radar)
    display.plot(
        "reflectivity_horizontal",
        colorbar_flag=True,
        mask_outside=True,
        mask_tuple=("reflectivity_horizontal", -100),
        ax=ax,
        gatefilter=gatefilter,
    )
    if outfile:
        fig.savefig(outfile)
    plt.close()


def test_radardisplay_cr_raster(outfile=None):
    radar = pyart.io.read_cfradial(pyart.testing.CFRADIAL_CR_RASTER_FILE)
    display = pyart.graph.RadarDisplay(radar)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    display.plot_cr_raster(ax=ax, target_range=478.0, el_limits=[-0.5, 2.5])
    if outfile:
        fig.savefig(outfile)
    plt.close()


def test_radardisplay_vpt_time(outfile=None):
    radar = pyart.io.read_cfradial(pyart.testing.CFRADIAL_PPI_FILE)
    pyart.util.to_vpt(radar)  # hack to make the data a VPT
    display = pyart.graph.RadarDisplay(radar)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    display.plot(
        "reflectivity_horizontal",
        colorbar_flag=True,
        time_axis_flag=True,
        edges=False,
        mask_tuple=("reflectivity_horizontal", -100),
        ax=ax,
    )
    if outfile:
        fig.savefig(outfile)
    plt.close()


def test_radardisplay_azimuth_to_rhi(outfile=None):
    radar = pyart.io.read_cfradial(pyart.testing.CFRADIAL_PPI_FILE)
    radar.antenna_transition = {"data": np.zeros(radar.nrays)}
    gatefilter = pyart.filters.GateFilter(radar)
    display = pyart.graph.RadarDisplay(radar)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    display.plot_azimuth_to_rhi(
        "reflectivity_horizontal",
        45.0,
        colorbar_flag=True,
        ax=ax,
        mask_tuple=("reflectivity_horizontal", -100),
        mask_outside=True,
        gatefilter=gatefilter,
    )
    if outfile:
        fig.savefig(outfile)
    plt.close()


# Tests of methods, these tests do not generate figures


def test_radardisplay_init():
    # test that a display object can be created with and without
    radar = pyart.io.read_cfradial(pyart.testing.CFRADIAL_PPI_FILE)
    radar.antenna_transition = {"data": np.zeros((40,))}
    display = pyart.graph.RadarDisplay(radar)
    assert display.antenna_transition is not None
    plt.close()


def test_radardisplay_plot_rhi_reverse():
    radar = pyart.io.read_cfradial(pyart.testing.CFRADIAL_RHI_FILE)
    display = pyart.graph.RadarDisplay(radar)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    display.plot("reflectivity_horizontal", 0, ax=ax, reverse_xaxis=True)
    plt.close()


def test_radardisplay_plot_azimuth_to_rhi_reverse(outfile=None):
    radar = pyart.io.read_cfradial(pyart.testing.CFRADIAL_PPI_FILE)
    display = pyart.graph.RadarDisplay(radar)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    display.plot_azimuth_to_rhi(
        "reflectivity_horizontal", 45.0, ax=ax, reverse_xaxis=True
    )
    plt.close()


def test_radardisplay_generate_filename():
    radar = pyart.io.read_cfradial(pyart.testing.CFRADIAL_PPI_FILE)
    display = pyart.graph.RadarDisplay(radar, shift=(0.1, 0.0))
    filename = display.generate_filename("test", 0)
    assert filename == "xsapr-sgp_test_00_20110520105416.png"

    filename = display.generate_filename("test", 0, datetime_format="%Y%m%d")
    assert filename == "xsapr-sgp_test_00_20110520.png"
    plt.close()


def test_radardisplay_plot_labels_errors():
    radar = pyart.io.read_cfradial(pyart.testing.CFRADIAL_PPI_FILE)
    display = pyart.graph.RadarDisplay(radar)
    # len(labels) != len(locations)
    pytest.raises(ValueError, display.plot_labels, ["a", "a"], [(0, 0)])
    # len(labels) != len(symbols)
    pytest.raises(
        ValueError, display.plot_labels, ["a"], [(0, 0)], symbols=["r+", "r+"]
    )
    # unknown scan_type
    display.scan_type = "foo"
    pytest.raises(ValueError, display.plot, "fake_field")
    plt.close()


def test_radardisplay_user_specified_labels():
    # test that labels are set when a user specifies them.
    radar = pyart.io.read_cfradial(pyart.testing.CFRADIAL_PPI_FILE)
    display = pyart.graph.RadarDisplay(radar)
    fig = plt.figure()
    ax = fig.add_subplot(111)

    display._set_ray_title("field", 0, "foo", ax)
    assert ax.get_title() == "foo"

    display._label_axes_ppi(("foo", "bar"), ax)
    assert ax.get_xlabel() == "foo"
    assert ax.get_ylabel() == "bar"

    display._label_axes_rhi(("spam", "eggs"), ax)
    assert ax.get_xlabel() == "spam"
    assert ax.get_ylabel() == "eggs"

    display._label_axes_ray(("baz", "qux"), "field", ax)
    assert ax.get_xlabel() == "baz"
    assert ax.get_ylabel() == "qux"

    display._label_axes_vpt(("nick", "nock"), False, ax)
    assert ax.get_xlabel() == "nick"
    assert ax.get_ylabel() == "nock"
    plt.close()


def test_radardisplay_loc_of_moving_radar():
    radar = pyart.testing.make_empty_ppi_radar(1, 1, 1)

    radar.latitude["data"] = np.array([35, 45])
    radar.longitude["data"] = np.array([75, 85])
    pytest.warns(UserWarning, pyart.graph.RadarDisplay, radar)
    with warnings.catch_warnings():
        warnings.simplefilter("ignore", category=UserWarning)
        display = pyart.graph.RadarDisplay(radar)
        assert_almost_equal(display.loc, (40, 80), 0)


def test_radardisplay_get_x_z():
    radar = pyart.testing.make_empty_ppi_radar(1, 1, 1)
    display = pyart.graph.RadarDisplay(radar)
    x, z = display._get_x_z(0, False, False)
    assert x.shape == (1, 1)
    assert z.shape == (1, 1)


def test_set_title():
    radar = pyart.testing.make_empty_ppi_radar(1, 1, 1)
    radar.fields["foo"] = {}
    display = pyart.graph.RadarDisplay(radar)
    fig = plt.figure()
    ax = fig.add_subplot(111)

    display._set_az_rhi_title("foo", 0, "special title", ax)
    assert ax.get_title() == "special title"


def test_radardisplay_misc():
    # misc methods which are not tested above
    radar = pyart.io.read_cfradial(pyart.testing.CFRADIAL_PPI_FILE)
    display = pyart.graph.RadarDisplay(radar)
    fig = plt.figure()
    ax = fig.add_subplot(111)

    # _set_vpt_title with a title
    display._set_vpt_title("foo_field", "title_string", ax)
    assert ax.get_title() == "title_string"

    # _generate_field_name method
    fn = pyart.graph.common.generate_field_name(radar, "reflectivity_horizontal")
    assert fn == "Equivalent reflectivity factor"

    display.fields["reflectivity_horizontal"].pop("standard_name")
    fn = pyart.graph.common.generate_field_name(radar, "reflectivity_horizontal")
    assert fn == "Reflectivity"

    display.fields["reflectivity_horizontal"].pop("long_name")
    fn = pyart.graph.common.generate_field_name(radar, "reflectivity_horizontal")
    assert fn == "Reflectivity horizontal"

    plt.close()


def test_radardisplay_get_colorbar_label():
    radar = pyart.io.read_cfradial(pyart.testing.CFRADIAL_PPI_FILE)
    display = pyart.graph.RadarDisplay(radar)

    # default is to base the label on the standard_name key
    assert (
        display._get_colorbar_label("reflectivity_horizontal")
        == "equivalent reflectivity factor (dBZ)"
    )

    # next is to look at the long_name
    del display.fields["reflectivity_horizontal"]["standard_name"]
    assert (
        display._get_colorbar_label("reflectivity_horizontal") == "Reflectivity (dBZ)"
    )

    # use the field if standard_name and long_name missing
    del display.fields["reflectivity_horizontal"]["long_name"]
    print(display._get_colorbar_label("reflectivity_horizontal"))
    assert (
        display._get_colorbar_label("reflectivity_horizontal")
        == "reflectivity horizontal (dBZ)"
    )

    # no units if key is missing
    del display.fields["reflectivity_horizontal"]["units"]
    print(display._get_colorbar_label("reflectivity_horizontal"))
    assert (
        display._get_colorbar_label("reflectivity_horizontal")
        == "reflectivity horizontal (?)"
    )
    plt.close()


def test_edge_time():
    radar = pyart.io.read(pyart.testing.MDV_PPI_FILE)
    times = pyart.util.datetimes_from_radar(radar)
    edge_time = pyart.graph.radardisplay._edge_time(times)
    assert edge_time[-1] == cftime.DatetimeGregorian(2011, 5, 20, 11, 6, 35, 933148)


if __name__ == "__main__":
    test_radardisplay_rhi("figure_radar_display_rhi.png")
    test_radardisplay_ppi("figure_radar_display_ppi.png")
    test_radardisplay_ray("figure_radar_display_ray.png")
    test_radardisplay_vpt("figure_radar_display_vpt.png")
    test_radardisplay_cr_raster("figure_radar_display_cr_raster.png")



""" Unit tests for spectra_calculations.py """

import pytest
from numpy.testing import assert_allclose

from pyart.retrieve import spectra_moments
from pyart.testing import make_target_spectra_radar

try:
    import xarray as xr  # noqa

    _XARRAY_AVAILABLE = True
except ImportError:
    _XARRAY_AVAILABLE = False


@pytest.mark.skipif(not _XARRAY_AVAILABLE, reason="Xarray is not installed.")
def test_spectra_moments():
    radar = make_target_spectra_radar()
    fields = spectra_moments(radar)
    assert len(fields.keys()) == 5
    assert_allclose(
        fields["reflectivity"]["data"][0, 0:5],
        [71.34390136, 71.34390136, 71.34390136, 71.34390136, 71.34390136],
        atol=1e-14,
    )
    assert_allclose(
        fields["velocity"]["data"][0, 0:5],
        [
            2.73382043e-16,
            2.73382043e-16,
            2.73382043e-16,
            2.73382043e-16,
            2.73382043e-16,
        ],
        atol=1e-14,
    )
    assert_allclose(
        fields["spectrum_width"]["data"][0, 0:5],
        [2.85047687, 2.85047687, 2.85047687, 2.85047687, 2.85047687],
        atol=1e-14,
    )
    assert_allclose(
        fields["skewness"]["data"][0, 0:5],
        [
            9.44294386e-17,
            9.44294386e-17,
            9.44294386e-17,
            9.44294386e-17,
            9.44294386e-17,
        ],
        atol=1e-14,
    )
    assert_allclose(
        fields["kurtosis"]["data"][0, 0:5],
        [2.79911197, 2.79911197, 2.79911197, 2.79911197, 2.79911197],
        atol=1e-14,
    )



""" Unit Tests for Py-ART's retrieve/comp_z.py module. """

import copy

import numpy as np
from numpy.testing import assert_array_equal, assert_equal

import pyart


def test_composite_z():
    # initalize test radar object
    radar = pyart.io.read(pyart.testing.NEXRAD_ARCHIVE_MSG31_FILE)
    ref_field = "reflectivity"
    gatefilter = pyart.filters.GateFilter(radar)
    gatefilter.exclude_transition()
    gatefilter.exclude_below("cross_correlation_ratio", 0.95)

    #################################
    # Trivial first test, all 0s
    #################################

    # make fake radar data with 0 everywhere
    z = np.zeros(radar.fields["reflectivity"]["data"].shape)
    z_old = radar.fields["reflectivity"]

    z_new = copy.deepcopy(z_old)
    z_new["data"] = z.astype("float32")
    radar.add_field("reflectivity", z_new, replace_existing=True)

    compz = pyart.retrieve.composite_reflectivity(
        radar, field=ref_field, gatefilter=gatefilter
    )
    assert_equal(compz.fields["composite_reflectivity"]["data"].max(), 0)

    #################################
    # Insert 1 layer of all 40 dBZ
    #################################

    z = np.zeros(radar.fields["reflectivity"]["data"].shape)

    # choose random sweep, it shouldnt matter which one.
    random_sweep = np.random.randint(0, radar.nsweeps)
    # loop over all measured sweeps
    for sweep in radar.sweep_number["data"]:
        if sweep == random_sweep:
            # get start and stop index numbers
            s_idx = radar.sweep_start_ray_index["data"][sweep]
            e_idx = radar.sweep_end_ray_index["data"][sweep] + 1
            z[s_idx:e_idx, :] = 40

    z_new = copy.deepcopy(z_old)
    z_new["data"] = z.astype("float32")
    radar.add_field("reflectivity", z_new, replace_existing=True)
    compz = pyart.retrieve.composite_reflectivity(
        radar, field=ref_field, gatefilter=gatefilter
    )
    assert_equal(compz.fields["composite_reflectivity"]["data"].max(), 40)

    ##########################################
    # Have dBZ increase according to range bin
    ##########################################

    z = np.zeros(radar.fields["reflectivity"]["data"].shape)

    # choose random sweep, it shouldnt matter which one.
    random_sweep = np.random.randint(0, radar.nsweeps)

    # loop over all measured sweeps
    for sweep in radar.sweep_number["data"]:
        if sweep == random_sweep:
            # get start and stop index numbers
            s_idx = radar.sweep_start_ray_index["data"][sweep]
            e_idx = radar.sweep_end_ray_index["data"][sweep] + 1
            z[s_idx:e_idx, :] = np.tile(
                np.arange(0, z.shape[1])[np.newaxis, :], (e_idx - s_idx, 1)
            )

    z_new = copy.deepcopy(z_old)
    z_new["data"] = z.astype("float32")
    radar.add_field("reflectivity", z_new, replace_existing=True)
    compz = pyart.retrieve.composite_reflectivity(
        radar, field=ref_field, gatefilter=gatefilter
    )

    # choose a random az
    random_az = np.random.randint(0, 720)
    assert_array_equal(
        compz.fields["composite_reflectivity"]["data"][random_az, :],
        np.arange(0, z.shape[1]),
    )
